@@ -446,10 +446,12 @@ class TestRestApiHandler(unittest.TestCase):
 class TestRestApiServer(unittest.TestCase):
 
     @patch('ssl.SSLContext.load_cert_chain', Mock())
+    @patch('ssl.SSLContext.set_ciphers', Mock())
     @patch('ssl.SSLContext.wrap_socket', Mock(return_value=0))
     @patch.object(BaseHTTPServer.HTTPServer, '__init__', Mock())
     def setUp(self):
-        self.srv = MockRestApiServer(Mock(), '', {'listen': '*:8008', 'certfile': 'a', 'verify_client': 'required'})
+        self.srv = MockRestApiServer(Mock(), '', {'listen': '*:8008', 'certfile': 'a', 'verify_client': 'required',
+                                                  'ciphers': '!SSLv1:!SSLv2:!SSLv3:!TLSv1:!TLSv1.1'})
 
     @patch.object(BaseHTTPServer.HTTPServer, '__init__', Mock())
     def test_reload_config(self):
@@ -488,3 +490,16 @@ class TestRestApiServer(unittest.TestCase):
         mock_accept.return_value = (newsock, '2')
         self.srv.socket = Mock()
         self.assertEqual(self.srv.get_request(), ((self.srv.socket, newsock), '2'))
+
+    @patch.object(MockRestApiServer, 'process_request', Mock(side_effect=RuntimeError))
+    def test_process_request_error(self):
+        mock_address = ('127.0.0.1', 55555)
+        mock_socket = Mock()
+        mock_ssl_socket = (Mock(), Mock())
+        for mock_request in (mock_socket, mock_ssl_socket):
+            with patch.object(
+                MockRestApiServer,
+                'get_request',
+                Mock(return_value=(mock_request, mock_address))
+            ):
+                self.srv._handle_request_noblock()
